<?php
/*
NOTICE OF LICENSE

This source file is subject to the NekloEULA that is bundled with this package in the file ICENSE.txt.

It is also available through the world-wide-web at this URL: http://store.neklo.com/LICENSE.txt

Copyright (c)  Neklo (http://store.neklo.com/)
*/


namespace Neklo\Scheduler\Controller\Adminhtml\Job;

use Neklo\Scheduler\Controller\Adminhtml;
use Magento\Backend\App\Action\Context;

abstract class MassAbstract extends Adminhtml\Action
{
    const ADMIN_RESOURCE = 'Neklo_Scheduler::edit';

    /**
     * Redirect url
     */
    const REDIRECT_URL = '*/job/index';

    /**
     * Field id
     */
    const ID_FIELD = 'selected';

    /**
     *
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     *
     * @var \Neklo\Scheduler\Model\JobFactory
     */
    private $factory;

    public function __construct(
        Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Registry $registry,
        \Neklo\Scheduler\Model\JobFactory $modelFactory,
        \Psr\Log\LoggerInterface $logger
    ) {
        parent::__construct($context, $resultPageFactory, $registry);
        $this->logger = $logger;
        $this->factory = $modelFactory;
    }

    public function execute()
    {
        $selected = $this->getSelected();
        try {
            if (!empty($selected)) {
                $this->setArray($selected);
            } else {
                $this->messageManager->addError(__('Please select item(s).'));
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            $this->messageManager->addError(__('Operation error'));
        }
        return $this->_redirect(static::REDIRECT_URL);
    }

    protected function setArray($items)
    {
        $errMsg = [];
        foreach ($items as $id) {
            $job = $this->factory->create();
            try {
                $job->load($id);
                if ($job->getId()) {
                    $this->itemOperation($job)->save();
                }
            } catch (\Exception $e) {
                $err = true;
                $this->logger->critical($e->getMessage());
                $errMsg[] = __($e->getMessage());
            }
        }
        if (!empty($err)) {
            $this->messageManager->addError(implode('<br>', $errMsg));
        }
    }

    protected function getSelected()
    {
        $selected = $this->getRequest()->getParam(self::ID_FIELD);
        $excluded = $this->getRequest()->getParam('excluded');
        if ($excluded !== null) {
            $selected = $this->factory->create()
                ->getCollection()
                ->addFieldToFilter('main_table.id', ['nin' => $excluded])
                ->getAllIds();
        }
        if (!is_array($selected)) {
            $selected = explode(',', $selected);
        }

        return $selected;
    }

    /**
     * Action on each item of mass action
     *
     * @param \Neklo\Scheduler\Model\Job $item
     *
     * @return \Neklo\Scheduler\Model\Job
     */
    abstract protected function itemOperation(\Neklo\Scheduler\Model\Job $item);

}
