'use strict';

const NODE_ENV = process.env.NODE_ENV || 'development';
const gulp = require('gulp');
const minify = require('gulp-minify');
const gulpif = require('gulp-if');
const concat = require('gulp-concat');
const stylus = require('gulp-stylus');
const sass = require('gulp-sass');
const path = require('path');
const autoprefixer = require('autoprefixer');
const postcss = require('gulp-postcss');
const imagemin = require('gulp-imagemin');
const jswrap = require('gulp-js-wrapper');
const uglifycss = require('gulp-uglifycss');
const fs = require('fs');
const spritesmith = require('gulp.spritesmith');

// config
const config = require('./config/config.json');

/**
 * Javascript
 */
gulp.task('js', () => {
    const vendor = config.main.src.js.vendor;
    const module = config.main.src.js.module;
    const minifyConfig = {
        ext: {
            min: '.min.js'
        }
    };

    // vendor
    gulp.src(vendor)
        .pipe(concat('vendor.js'))
        .pipe(gulpif(NODE_ENV == 'production', minify(minifyConfig)))
        .pipe(gulpif(!!config.main.src.amd.vendor.length, jswrap({
            opener: "require([" + config.main.src.amd.vendor + "], function(){",
            closer: "});"
        })))
        .pipe(gulp.dest(config.main.dest.js));

    // module
    gulp.src(module)
        .pipe(concat('module.js'))
        .pipe(gulpif(NODE_ENV == 'production', minify(minifyConfig)))
        .pipe(gulpif(!!config.main.src.amd.module.length, jswrap({
            opener: "require([" + config.main.src.amd.module + "], function(){",
            closer: "});"
        })))
        .pipe(gulp.dest(config.main.dest.js));

    // all
    gulp.src([].concat(vendor, module))
        .pipe(concat('all.js'))
        .pipe(gulpif(NODE_ENV == 'production', minify(minifyConfig)))
        .pipe(gulpif(!!config.main.src.amd.all.length, jswrap({
            opener: "require([" + config.main.src.amd.all + "], function(){",
            closer: "});"
        })))
        .pipe(gulp.dest(config.main.dest.js));

    // no concat
    gulp.src(config.main.src.js.noconcat)
        .pipe(gulpif(NODE_ENV == 'production', minify(minifyConfig)))
        .pipe(gulp.dest(config.main.dest.js));
});
/**
 * Css
 */
gulp.task('stylus', () => {
    const processors = [
        autoprefixer({ browsers: ['last 100 version'] })
    ];

    fs.stat(path.join(__dirname, config.main.src.stylus).slice(0, -6), (err, start) => {
        if(err) return;

        gulp.src(config.main.src.stylus)
            .pipe(stylus({
                compress: NODE_ENV == 'production'
            }))
            .pipe(postcss(processors))
            .pipe(gulp.dest(config.main.dest.stylus))
    });
});

gulp.task('sass', function () {
    const processors = [
        autoprefixer({ browsers: ['last 100 version'] })
    ];

    fs.stat(path.join(__dirname, config.main.src.sass).slice(0, -6), (err, start) => {
        if(err) return;

        gulp.src(config.main.src.sass)
            .pipe(sass().on('error', sass.logError))
            .pipe(postcss(processors))
            .pipe(gulpif(NODE_ENV == 'production', uglifycss()))
            .pipe(gulp.dest(config.main.dest.sass));
    });


});

/**
 * fonts
 */
gulp.task('fonts', () => {
    gulp.src(config.main.src.fonts)
        .pipe(gulp.dest(config.main.dest.fonts))
});

/**
 * watch
 */
gulp.task('watch', function() {
    gulp.watch(config.main.watch.stylus, ['css']);
    gulp.watch(config.main.sprite.src, ['sprite']);
    gulp.watch(config.main.watch.js, function (event) {
        gulp.run('js');
    });
});

/**
 * default
 */
gulp.task('default', ['js', 'stylus', 'sass', 'fonts',  "watch"]);

/**
 * Build dev
 */
gulp.task('dev', ['js', 'stylus', 'sass', 'fonts' ]);

/**
 * Build prod
 */
gulp.task('prod', ['js', 'stylus', 'sass', 'fonts']);