(function (factory) {
    if (typeof define === "function" && define.amd) {
        define([
            "jquery",
            'mage/template',
            "mage/mage"
        ], factory);
    } else {
        factory(jQuery);
    }
}(function ($, mageTemplate, alert) {
    "use strict";

    $.widget('neklo.timeline', {
        options: {
            stride : 60*60*1000 //in milisecconds
        },

        /**
         * @private
         */
        _create: function()
        {
            this.jobs = {};
            this.tooltip;
            this.limits = [
                this.options.timeStart * 1000,
                this.options.timeEnd * 1000
            ];
            if (this.limits[0] > this.limits[1]) {
                var date  = this.limits[0];
                this.limits[0] = this.limits[1];
                this.limits[1] = date;
            }
            var that = this;
            if(this.element.find('.jobs').length == 0) return;
            this._startLoad(0);
            this.eventTpl = mageTemplate('#event-template');//process timeline
            this.processTpl = mageTemplate('#detail-template');//tooltip detail
            this.processbarTpl = mageTemplate('#process-bar-template');//timeline bar
            this.hoursTpl = mageTemplate('#hours-template');//hours
            this.jobsTpl = mageTemplate('#jobs-template');//names
            setTimeout(function(){
                that._prepareData();
                that._initTimeline();
                that._initTooltip();
                that._endLoad(1);
                that._setupJobFilter();
            }, 1000);
        },

        _initTooltip : function()
        {
            var that = this;
            this.element.find('.event .process-bar').bind('mouseenter', function(event){
                that.showToolTip(this, event);
            }).bind('mouseout', function(event){
                that.hideToolTip(this, event);
            });

            this.tooltip = $('<div class="timeline-tooltip" style="display:none;">');
            $('body').append(this.tooltip);
            this.overFunction = function(){
                var timeout = that.tooltip.attr('data-hide');
                //console.log(timeout);
                if (timeout) {
                    clearTimeout(timeout*1);
                    that.tooltip.removeAttr('data-hide');
                }
            }
            this.outFunction = function(event) {
                that.hideToolTip(this, event);
            }
            this.tooltip
                .on('mouseover', this.overFunction)
                .on('mouseout', this.outFunction);
        },

        /**
         * @private
         */
        _initTimeline: function()
        {
            var idx = 0;//process index
            var count = Object.keys(this.jobs).length+1;//+1 for hours block
            var hourPanel = this.element.find('.row.hours');
            hourPanel.hide();
            var date;
            var timeSpan = (this.limits[1] - this.limits[0]);
            var slots = Math.ceil(timeSpan / this.options.stride);
            var min, mon;
            var hourOut = this.hoursTpl;
            for(var i = this.limits[0]; i <= this.limits[1]; i += this.options.stride) {
                date = new Date(i);
                min = date.getMinutes();
                mon = date.getMonth()+1;
                min = min < 10 ? '0'+min : min;
                mon = mon < 10 ? '0'+mon : mon;
                hourPanel.append(hourOut({
                    data : {
                        style : 'width:'+ (100/slots)+'%',
                        date : date.getDate()+'.'+mon+'.',
                        year : date.getFullYear(),
                        time : date.getHours()+':'+min
                    }
                }));
            }
            hourPanel.css({ diplay : 'inline'}).show();
            idx++;
            this._startLoad(idx/count);

            var width = hourPanel.width();
            var jobDetailTpl = this.jobsTpl;//names
            var jobEventTpl = this.eventTpl;//timeline block
            var jobData = this.element.find('.timeline-container');
            var jobNames = this.element.find('.timeline-panel .jobs');

            for(var i in this.jobs) {
                if(!this.jobs.hasOwnProperty(i)) continue;

                jobNames.append(jobDetailTpl({
                    data : {
                        code : i,
                        name : i
                    }
                }));

                jobData.append(jobEventTpl({
                    data : {
                        code : i,
                        class : '',
                        style : 'width:'+width+'px',
                        bar : this._addData(this.jobs[i], timeSpan, width)
                    }
                }));
                idx++;
                this._startLoad(idx/count);
            }
        },

        _startLoad: function(val)
        {
            var loader = $('body').data().loader;
            if (val == 0) {
                $('body').loader('show');
                this.initText = loader.element.find('.popup-inner').text();
            } else {// update
                if (loader) {
                    loader.element.find('.popup-inner').text(this.initText + (val*100)+'%');
                }
            }
        },
        _endLoad: function(end)
        {
            $('body').loader('hide');
        },
        _addData: function(data, timeSpan, width)
        {
            var that = this;
            var event, start, stop, toStart, last ,w;
            var times = [];
            var counts = [];
            var idx;
            var outData = '';
            for(var i = 0; i < data.length; i++) {
                event = data[i];
                start = Date.parse(event.executed_at);
                stop = Date.parse(event.finished_at);
                toStart = Date.parse(event.scheduled_at);
                idx = this._isThere(times, toStart);
                if (idx < 0){
                    times.push(toStart);
                    counts.push(0);
                    idx = 0;
                } else {
                    counts[idx]++;
                }
                last = (toStart - this.limits[0])/(timeSpan/100);
                var style = 'left:'+ last+'%;'+
                    'top:'+ (counts[idx]*4)+'px;';
                if (i > 0) {
                    style += 'border-top-width:1px;';
                }
                w = (stop - start)/width;
                if(w < 1) w = 3;
                if (!isNaN(start) && !isNaN(stop)) {
                    style += 'height:'+ (39-(counts[idx]*4))+'px;'+
                        'margin-left:'+ ((start - toStart)/width)+'px;'+
                        'width:'+ w+'px;';
                } else {
                    if (event.status !== 'error') {
                    }
                }

                var tpl = this.processbarTpl({
                    data : {
                        values : JSON.stringify(event),
                        style : style,
                        class : event.status
                    }
                });
                outData += tpl;
            }
            return outData;
        },
        /**
         * checks, if there is timestamp within margin and returns index. -1 default
         * @returns index
         */
        _isThere: function(stampArray, stamp, margin)
        {
            var half1 = stamp - (margin/2);
            var half2 = stamp + (margin/2);
            for(var i in stampArray) {
                if (stampArray[i] >= half1 && stampArray[i] <= half2 ) {
                    return i;
                }
            }
            return -1;
        },

        showToolTip : function(element, event)
        {
            var el = $(element);
            var data = JSON.parse(el.attr('data-values'));

            if (this.tooltip.attr('data-hide')) {
                clearTimeout(this.tooltip.attr('data-hide')*1);
                this.tooltip.removeAttr('data-hide');
            }
            this.tooltip.html(this.processTpl({data : data}));
            this.tooltip.show();
            this.tooltip.css('top', (event.pageY+5)+'px');
            this.tooltip.css('left', (event.pageX+5)+'px');
            //check window borders
            if (window.innerWidth - (event.clientX + this.tooltip.width()) < 0) {
                this.tooltip.css('left', (event.pageX - this.tooltip.width() - 5)+'px');
            }
            if (window.innerHeight - (event.clientY + this.tooltip.height()) < 0) {
                this.tooltip.css('top', (event.pageY - this.tooltip.height() - 5)+'px');
            }
            //add events to new table
            this.tooltip.find('.process > table')
                .on('mouseover', this.overFunction)
                .on('mouseout', this.outFunction);
        },
        hideToolTip : function(element, event)
        {
            var el = $(element);
            var pos = el.position();
            if (
                (event.pageX >= pos.left && event.pageX <= pos.left + el.width()) &&
                (event.pageY >= pos.top && event.pageY <= pos.top + el.height())
            ) {
                return;
            } else {
                var that = this;
                var timeout = setTimeout(function(){
                    that.tooltip.hide();
                }, 1000);
                this.tooltip.attr('data-hide', timeout);
            }
        },

        _prepareData: function()
        {
            var item;
            var that = this;
            var minmax = function(value){
                if (!isNaN(value)) {
                    if (value < that.limits[0]) {
                        that.limits[0] = value;
                    } else if(value > that.limits[1]) {
                        that.limits[1] = value;
                    }
                }
            }
            var names = [];
            for(var i in this.options.dataSource) {
                item = this.options.dataSource[i];
                if (names.indexOf(item.job_code) === -1) {
                    names.push(item.job_code);
                }
                if (!this.jobs[item.job_code]) {
                    this.jobs[item.job_code] = [];
                }
                this.jobs[item.job_code].push(item);
                minmax(Date.parse(item.created_at));
                minmax(Date.parse(item.executed_at));
                minmax(Date.parse(item.finished_at));
                minmax(Date.parse(item.scheduled_at));
            }
            names = names.sort().reverse();
            var tmpJobs = {};
            for(var i in names) {
                tmpJobs[names[i]] = this.jobs[names[i]];
            }
            this.jobs = tmpJobs;

            this.limits[0] -= this.options.stride;
            this.limits[1] += this.options.stride;
        },

        _setupJobFilter : function() {
            var that = this;
            var el = this.element.find('.jobs .row:first-child');
            var select = '<select class="job-select"><option value="">&nbsp;</option>';
            for(var i in this.jobs) {
                select += '<option value="'+i+'">'+i+'</option>';
            }
            el.append(select+'</select>');
            el.find('select').on('change', function(){
                that._updateJobFilter($(this).val());
            })
        },
        _updateJobFilter : function(value) {
            for(var i in this.jobs) {
                if(!this.jobs.hasOwnProperty(i)) continue;
                if(value && i != value) {
                    this.element.find('.row[data-code="'+i+'"]').hide();
                } else {
                    this.element.find('.row[data-code="'+i+'"]').show();
                }
            }
        }
    });

    return $.neklo.timeline;
}));